@@ -1,7 +1,7 @@
 /***************************************************************************
  *                                                                         *
  *    LIBDSK: General floppy and diskimage access library                  *
- *    Copyright (C) 2001  John Elliott <jce@seasip.demon.co.uk>            *
+ *    Copyright (C) 2001  John Elliott <seasip.webmaster@gmail.com>            *
  *                                                                         *
  *    This library is free software; you can redistribute it and/or        *
  *    modify it under the terms of the GNU Library General Public          *
@@ -278,7 +278,7 @@ dsk_err_t nwasp_getgeom(DSK_DRIVER *self, DSK_GEOMETRY *geom)
                                          * mbee floppies) are untested. */
         geom->dg_rwgap     = 0x0C;
         geom->dg_fmtgap    = 0x17;
-        geom->dg_fm        = 0;
+        geom->dg_fm        = RECMODE_MFM;
         geom->dg_nomulti   = 0;
         return DSK_ERR_OK;
 }
