@@ -1,7 +1,7 @@
 /***************************************************************************
  *                                                                         *
  *    LIBDSK: General floppy and diskimage access library                  *
- *    Copyright (C) 2005,2008,2010-11  John Elliott <jce@seasip.demon.co.uk> *
+ *    Copyright (C) 2005,2008,2010-11  John Elliott <seasip.webmaster@gmail.com> *
  *                                                                         *
  *    This library is free software; you can redistribute it and/or        *
  *    modify it under the terms of the GNU Library General Public          *
@@ -1906,7 +1906,7 @@ dsk_err_t rcpmfs_write(DSK_DRIVER *self, const DSK_GEOMETRY *geom,
 	dir_sectors = rcpmfs_secperblock(rcself) * rcself->rc_dirblocks;
 
 /* Directory writes have to follow through to the underlying directory */
-	if (buffer && lsect >= 0 && lsect < dir_sectors)
+	if (buffer && lsect < dir_sectors)
 	{
 		dir_entries  = (rcself->rc_geom.dg_secsize / 32);   
 		dir_first	= lsect * dir_entries;
