@@ -1,7 +1,7 @@
 /***************************************************************************
  *                                                                         *
  *    LIBDSK: General floppy and diskimage access library                  *
- *    Copyright (C) 2001  John Elliott <jce@seasip.demon.co.uk>            *
+ *    Copyright (C) 2001  John Elliott <seasip.webmaster@gmail.com>            *
  *                                                                         *
  *    This library is free software; you can redistribute it and/or        *
  *    modify it under the terms of the GNU Library General Public          *
@@ -37,6 +37,19 @@ LDPUBLIC32 dsk_err_t LDPUBLIC16 dsk_ptread(DSK_DRIVER *self, const DSK_GEOMETRY
 	if (dc->dc_tread) 
 	{
 		err = (dc->dc_tread)(self,geom,buf,cylinder,head);	
+
+		/* If set to store bytes complemented, flip them as they come
+		 * out. (If we are using the dsk_pread() fallback, the 
+		 * complement will take place in dsk_pread() ) */
+		if (geom->dg_fm & RECMODE_COMPLEMENT)
+		{
+			for (sec = 0; sec < geom->dg_sectors * geom->dg_secsize;
+				sec++)
+			{
+				((char *)buf)[sec] = ~((char *)buf)[sec];
+			}
+		}
+
 		if (err != DSK_ERR_NOTIMPL) return err;
 	}
 
