@@ -1,7 +1,7 @@
 /***************************************************************************
  *                                                                         *
  *    LIBDSK: General floppy and diskimage access library                  *
- *    Copyright (C) 2005,6  John Elliott <jce@seasip.demon.co.uk>          *
+ *    Copyright (C) 2005,6  John Elliott <seasip.webmaster@gmail.com>          *
  *                                                                         *
  *    This library is free software; you can redistribute it and/or        *
  *    modify it under the terms of the GNU Library General Public          *
@@ -124,7 +124,9 @@ typedef enum
 	VT_SIZE,
 	VT_RATE,
 	VT_GAP,
-	VT_BOOLEAN
+	VT_BOOLEAN,
+	VT_RECMODE,
+	VT_COMPLEMENT
 } TYPE;
 
 /* And this describes one variable */
@@ -147,7 +149,8 @@ static VARIABLE vars[] =
 	{ VT_RATE,    &dg.dg_datarate,  "datarate" },
 	{ VT_GAP,     &dg.dg_rwgap,     "rwgap" },
 	{ VT_GAP,     &dg.dg_fmtgap,    "fmtgap" },
-	{ VT_BOOLEAN, &dg.dg_fm,        "fm" },
+	{ VT_RECMODE, &dg.dg_fm,        "recmode" },
+	{ VT_COMPLEMENT,  &dg.dg_fm,        "complement" },
 	{ VT_BOOLEAN, &dg.dg_nomulti,   "nomulti" },
 	{ VT_BOOLEAN, &dg.dg_noskip,    "noskip" },
 };
@@ -216,6 +219,20 @@ void set_var(VARIABLE *var, const char *value)
 					  break;
 			}
 			break;
+		case VT_RECMODE:
+			switch(toupper(value[0]))
+			{
+				case 'M': *((int *)(var->data)) &= ~RECMODE_MASK;
+					  *((int *)(var->data)) |= RECMODE_MFM;
+					  break;
+				case 'F': *((int *)(var->data)) &= ~RECMODE_MASK;
+					  *((int *)(var->data)) |= RECMODE_FM;
+					  break;
+				default:  printf("Recording mode must be FM or MFM\n");
+					  break;
+			}
+			break;
+
 		case VT_GAP:
 			r = sscanf(value, "%x", &hexv);
 			if (!r)
@@ -236,6 +253,17 @@ void set_var(VARIABLE *var, const char *value)
 					break;
 			}
 			break;
+		case VT_COMPLEMENT:
+			switch(toupper(value[0]))
+			{
+				case '1': case 'Y':
+					*((int *)(var->data)) |= RECMODE_COMPLEMENT;
+					break;
+				case '0': case 'N':
+					*((int *)(var->data)) &= ~RECMODE_COMPLEMENT;
+					break;
+			}
+			break;
 		default:
 			printf("Internal error: unsupported variable type %d\n",
 				var->type);
@@ -268,10 +296,22 @@ void print_var(VARIABLE *var)
 					      break;
 			}
 			break;
+		case VT_RECMODE:
+			switch(*((int *)(var->data)) & RECMODE_MASK)
+			{
+				case RECMODE_MFM: printf("MFM"); break;
+				case RECMODE_FM:  printf("FM"); break;
+				default:      printf("Unknown %d",
+						*((int *)(var->data)) & RECMODE_MASK);
+					      break;
+			}
+			break;
 		case VT_GAP:
 			printf("%x", *((dsk_gap_t *)(var->data))); break;
 		case VT_BOOLEAN:
 			printf("%c", *((int *)(var->data)) ? 'Y' : 'N'); break;
+		case VT_COMPLEMENT:
+			printf("%c", (*((int *)(var->data)) & RECMODE_COMPLEMENT) ? 'Y' : 'N'); break;
 		default:
 			printf("Internal error: unsupported variable type %d\n",
 				var->type);
@@ -541,8 +581,8 @@ static void dump(char *cmd, int showhex, int showascii)
 		if (from == -1) from = 0;
 		upto = 1 + get_hex(t);
 	}
-	if (from < 0  || from >= dg.dg_secsize) from = 0;
-	if (upto <= 0 || upto >  dg.dg_secsize) upto = dg.dg_secsize;
+	if (from < 0  || (unsigned)from >= dg.dg_secsize) from = 0;
+	if (upto <= 0 || (unsigned)upto >  dg.dg_secsize) upto = dg.dg_secsize;
 	if (from > upto) { n = from; from = upto-1; upto = n+1; }
 	for (n = from; n < upto; n = ((n / 16)+1)*16)
 	{
@@ -793,7 +833,7 @@ dsk_err_t save_yank(const char *s)
 	for (b0 = yank_chain; b0 != NULL; b0 = b0->next)
 	{
 		int res = fwrite(b0->data, 1, b0->length, fp);
-		if (res < b0->length)
+		if (res < (int)b0->length)
 		{
 			printf("Write error on file: %s\n", s);
 			fclose(fp);
@@ -900,7 +940,7 @@ dsk_err_t obey(char *cmd)
 		case 'N':
 			return new_geometry(cmd+1);
 		case 'O':
-			for (value = 0; value < dg.dg_secsize; value++)
+			for (value = 0; value < (int)dg.dg_secsize; value++)
 			{
 				secbuf[value] ^= 0xFF;
 			}
@@ -993,12 +1033,18 @@ void show_geometry(void)
 	printf("Data rate:\t");
 	switch(dg.dg_datarate)
 	{
-		case RATE_SD: printf(" 250\n"); break;
-		case RATE_DD: printf(" 300\n"); break;
-		case RATE_HD: printf(" 500\n"); break;
-		case RATE_ED: printf("1000\n"); break;
+		case RATE_SD: printf(" 250"); break;
+		case RATE_DD: printf(" 300"); break;
+		case RATE_HD: printf(" 500"); break;
+		case RATE_ED: printf("1000"); break;
 	}
-	printf("Encoding:\t %s\n\n", dg.dg_fm ? " FM" :"MFM");
+	printf("\nEncoding:\t ");
+	switch(dg.dg_fm & RECMODE_MASK)
+	{
+		case RECMODE_MFM: printf("MFM"); break;	
+		case RECMODE_FM:  printf(" FM"); break;	
+	}
+	printf("\n\n");
 }
 
 
@@ -1130,7 +1176,7 @@ dsk_err_t search(const char *arg)
 			showts();
 			return err;
 		}
-		for (n = 0; n < dg.dg_secsize - buflen; n++)
+		for (n = 0; n < (int)(dg.dg_secsize - buflen); n++)
 		{
 			if (!memcmp(secbuf + n, buf, buflen))
 			{
@@ -1177,8 +1223,8 @@ int get_range(const char *arg, int *from, int *upto)
 		printf("Cannot parse as xx or xx-xx: %s\n", arg);
 		return 0;
 	}
-	if (*from >= dg.dg_secsize) *from = dg.dg_secsize - 1;
-	if (*upto >= dg.dg_secsize) *upto = dg.dg_secsize - 1;
+	if (*from >= (int)dg.dg_secsize) *from = (int)dg.dg_secsize - 1;
+	if (*upto >= (int)dg.dg_secsize) *upto = (int)dg.dg_secsize - 1;
 	if (*from > *upto)
 	{
 		int tmp = *from; *from = *upto; *upto = tmp;
@@ -1217,13 +1263,13 @@ dsk_err_t change(const char *arg, int ascii)
 	{
 		for (n = 0; n < buflen; n++)
 		{
-			if (n + from >= dg.dg_secsize) break;
+			if ((unsigned)(n + from) >= dg.dg_secsize) break;
 			secbuf[n+from] = buf[n];
 		}
 	}
 	else for (n = from; n <= upto; n++)
 	{
-		if (n >= dg.dg_secsize) break;
+		if ((unsigned)n >= dg.dg_secsize) break;
 		secbuf[n] = buf[ (n-from) % buflen ];
 	}
 	dsk_free(buf);
